@@ -1,4 +1,3 @@
-from distutils.version import LooseVersion
 import functools
 import django
 from django.conf import settings
@@ -10,7 +9,7 @@ else:
     from django.contrib.admin.util import (flatten_fieldsets, unquote,
         get_deleted_objects)
 from django.contrib.contenttypes.models import ContentType
-from django.core.exceptions import ValidationError, PermissionDenied
+from django.core.exceptions import PermissionDenied
 from django.core.urlresolvers import reverse
 from django.db import router, transaction
 from django.forms.models import model_to_dict
@@ -18,7 +17,7 @@ if django.VERSION >= (1, 7):
     from django.forms.utils import ErrorList
 else:
     from django.forms.util import ErrorList
-from django.http import Http404, HttpResponseRedirect
+from django.http import Http404, HttpResponseRedirect, QueryDict
 from django.shortcuts import render_to_response
 from django.template import TemplateDoesNotExist
 from django.template.context import RequestContext
@@ -28,13 +27,12 @@ from django.utils.functional import curry
 from django.utils.translation import ugettext_lazy as _, get_language
 from functools import update_wrapper
 from hvad.compat.force_unicode import force_unicode
-from hvad.compat.urls import urlencode
+from hvad.compat.urls import urlencode, urlparse
 from hvad.forms import TranslatableModelForm, translatable_inlineformset_factory, translatable_modelform_factory
 from hvad.utils import get_cached_translation, get_translation
 from hvad.manager import FALLBACK_LANGUAGES
 
 
-NEW_GET_DELETE_OBJECTS = LooseVersion(django.get_version()) >= LooseVersion('1.3')
 atomic = (transaction.atomic if django.VERSION >= (1, 6) else
           transaction.commit_on_success)
 
@@ -110,11 +108,11 @@ class TranslatableModelAdminMixin(object):
 
     def get_language_tabs(self, request, available_languages):
         tabs = []
-        get = dict(request.GET)
+        get = request.GET.copy()
         language = self._language(request)
         for key, name in settings.LANGUAGES:
-            get.update({'language': key})
-            url = '%s?%s' % (request.path, urlencode(get))
+            get['language'] = key
+            url = '%s?%s' % (request.path, get.urlencode())
             if language == key:
                 status = 'current'
             elif key in available_languages:
@@ -150,11 +148,11 @@ class TranslatableAdmin(ModelAdmin, TranslatableModelAdminMixin):
 
 
     def get_urls(self):
-        try:
+        if django.VERSION >= (1, 4):
             from django.conf.urls import patterns, url
-        except ImportError:
-            from django.conf.urls.defaults import patterns, url            
-        
+        else: #pragma: no cover
+            from django.conf.urls.defaults import patterns, url
+
         urlpatterns = super(TranslatableAdmin, self).get_urls()
 
         def wrap(view):
@@ -227,6 +225,16 @@ class TranslatableAdmin(ModelAdmin, TranslatableModelAdminMixin):
         context['allow_deletion'] = len(available_languages) > 1
         context['language_tabs'] = self.get_language_tabs(request, available_languages)
         context['base_template'] = self.get_change_form_base_template()
+
+        # Ensure form action url carries over tab language
+        qs_language = request.GET.get('language')
+        if qs_language:
+            form_url = urlparse(form_url or request.get_full_path())
+            query = QueryDict(form_url.query, mutable=True)
+            if 'language' not in query:
+                query['language'] = qs_language
+            form_url = form_url._replace(query=query.urlencode()).geturl()
+
         return super(TranslatableAdmin, self).render_change_form(request,
                                                                   context,
                                                                   add, change,
@@ -272,13 +280,8 @@ class TranslatableAdmin(ModelAdmin, TranslatableModelAdminMixin):
         # will also be deleted.
         
         protected = False
-        if NEW_GET_DELETE_OBJECTS:
-            (deleted_objects, perms_needed, protected) = get_deleted_objects(
-                [obj], translations_model._meta, request.user, self.admin_site, using)
-        else: # pragma: no cover
-            (deleted_objects, perms_needed) = get_deleted_objects(
-                [obj], translations_model._meta, request.user, self.admin_site)
-        
+        deleted_objects, perms_needed, protected = get_deleted_objects(
+            [obj], translations_model._meta, request.user, self.admin_site, using)
         
         lang = get_language_name(language_code) 
             
@@ -445,7 +448,10 @@ class TranslatableInlineModelAdmin(InlineModelAdmin, TranslatableModelAdminMixin
         return translatable_inlineformset_factory(language, self.parent_model, self.model, **defaults)
 
     def get_urls(self):
-        from django.conf.urls.defaults import patterns, url
+        if django.VERSION >= (1, 4):
+            from django.conf.urls import patterns, url
+        else: #pragma: no cover
+            from django.conf.urls.defaults import patterns, url
 
         urlpatterns = super(InlineModelAdmin, self).get_urls()
 
